@@ -98,10 +98,11 @@ pub(crate) fn get_builtin_filters() -> BTreeMap<&'static str, BoxedFilter> {
     rv.insert("safe", BoxedFilter::new(safe));
     rv.insert("escape", BoxedFilter::new(escape));
     rv.insert("e", BoxedFilter::new(escape));
-    #[cfg(feature = "builtin_filters")]
+    #[cfg(feature = "builtins")]
     {
         rv.insert("lower", BoxedFilter::new(lower));
         rv.insert("upper", BoxedFilter::new(upper));
+        rv.insert("title", BoxedFilter::new(title));
         rv.insert("replace", BoxedFilter::new(replace));
         rv.insert("length", BoxedFilter::new(length));
         rv.insert("count", BoxedFilter::new(length));
@@ -110,7 +111,14 @@ pub(crate) fn get_builtin_filters() -> BTreeMap<&'static str, BoxedFilter> {
         rv.insert("trim", BoxedFilter::new(trim));
         rv.insert("join", BoxedFilter::new(join));
         rv.insert("default", BoxedFilter::new(default));
+        rv.insert("round", BoxedFilter::new(round));
+        rv.insert("abs", BoxedFilter::new(abs));
+        rv.insert("first", BoxedFilter::new(first));
+        rv.insert("last", BoxedFilter::new(last));
         rv.insert("d", BoxedFilter::new(default));
+        rv.insert("list", BoxedFilter::new(list));
+        rv.insert("batch", BoxedFilter::new(batch));
+        rv.insert("slice", BoxedFilter::new(slice));
         #[cfg(feature = "json")]
         {
             rv.insert("tojson", BoxedFilter::new(tojson));
@@ -143,30 +151,49 @@ pub fn escape(_state: &State, v: Value) -> Result<Value, Error> {
     }
 }
 
-#[cfg(feature = "builtin_filters")]
+#[cfg(feature = "builtins")]
 mod builtins {
     use super::*;
 
     use crate::error::ErrorKind;
     use crate::utils::matches;
-    use crate::value::{Primitive, ValueKind};
-    use std::cmp::Ordering;
+    use crate::value::{ValueKind, ValueRepr};
     use std::fmt::Write;
+    use std::mem;
 
     /// Converts a value to uppercase.
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn upper(_state: &State, v: String) -> Result<String, Error> {
         Ok(v.to_uppercase())
     }
 
     /// Converts a value to lowercase.
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn lower(_state: &State, v: String) -> Result<String, Error> {
         Ok(v.to_lowercase())
     }
 
+    /// Converts a value to title case.
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
+    pub fn title(_state: &State, v: String) -> Result<String, Error> {
+        let mut rv = String::new();
+        let mut capitalize = true;
+        for c in v.chars() {
+            if c.is_ascii_punctuation() || c.is_whitespace() {
+                rv.push(c);
+                capitalize = true;
+            } else if capitalize {
+                write!(rv, "{}", c.to_uppercase()).unwrap();
+                capitalize = false;
+            } else {
+                write!(rv, "{}", c.to_lowercase()).unwrap();
+            }
+        }
+        Ok(rv)
+    }
+
     /// Does a string replace.
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn replace(_state: &State, v: String, from: String, to: String) -> Result<String, Error> {
         Ok(v.replace(&from, &to))
     }
@@ -174,7 +201,7 @@ mod builtins {
     /// Returns the "length" of the value
     ///
     /// By default this filter is also registered under the alias `count`.
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn length(_state: &State, v: Value) -> Result<Value, Error> {
         v.len().map(Value::from).ok_or_else(|| {
             Error::new(
@@ -185,22 +212,30 @@ mod builtins {
     }
 
     /// Dict sorting functionality.
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn dictsort(_state: &State, v: Value) -> Result<Value, Error> {
-        let mut pairs = v.try_into_pairs()?;
-        pairs.sort_by(|a, b| a.0.partial_cmp(&b.0).unwrap_or(Ordering::Equal));
+        let mut pairs = match v.0 {
+            ValueRepr::Map(ref v) => v.iter().collect::<Vec<_>>(),
+            _ => {
+                return Err(Error::new(
+                    ErrorKind::ImpossibleOperation,
+                    "cannot convert value into pair list",
+                ))
+            }
+        };
+        pairs.sort_by(|a, b| a.0.cmp(b.0));
         Ok(Value::from(
             pairs
                 .into_iter()
-                .map(|(k, v)| vec![k, v])
+                .map(|(k, v)| vec![Value::from(k.clone()), v.clone()])
                 .collect::<Vec<_>>(),
         ))
     }
 
     /// Reverses a list or string
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn reverse(_state: &State, v: Value) -> Result<Value, Error> {
-        if let Some(Primitive::Str(s)) = v.as_primitive() {
+        if let Some(s) = v.as_str() {
             Ok(Value::from(s.chars().rev().collect::<String>()))
         } else if matches!(v.kind(), ValueKind::Seq) {
             let mut v = v.try_into_vec()?;
@@ -215,7 +250,7 @@ mod builtins {
     }
 
     /// Trims a value
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn trim(_state: &State, s: String, chars: Option<String>) -> Result<String, Error> {
         match chars {
             Some(chars) => {
@@ -227,7 +262,7 @@ mod builtins {
     }
 
     /// Joins a sequence by a character
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn join(_state: &State, val: Value, joiner: Option<String>) -> Result<String, Error> {
         if val.is_undefined() || val.is_none() {
             return Ok(String::new());
@@ -235,7 +270,7 @@ mod builtins {
 
         let joiner = joiner.as_ref().map_or("", |x| x.as_str());
 
-        if let Some(Primitive::Str(s)) = val.as_primitive() {
+        if let Some(s) = val.as_str() {
             let mut rv = String::new();
             for c in s.chars() {
                 if !rv.is_empty() {
@@ -268,7 +303,7 @@ mod builtins {
     /// Checks if a string starts with another string.
     ///
     /// By default this filter is also registered under the alias `d`.
-    #[cfg_attr(docsrs, doc(cfg(feature = "builtin_filters")))]
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn default(_: &State, value: Value, other: Option<Value>) -> Result<Value, Error> {
         Ok(if value.is_undefined() {
             other.unwrap_or_else(|| Value::from(""))
@@ -277,13 +312,174 @@ mod builtins {
         })
     }
 
+    /// Returns the absolute value of a number.
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
+    pub fn abs(_: &State, value: Value) -> Result<Value, Error> {
+        match value.0 {
+            ValueRepr::I64(x) => Ok(Value::from(x.abs())),
+            ValueRepr::I128(x) => Ok(Value::from(x.abs())),
+            ValueRepr::F64(x) => Ok(Value::from(x.abs())),
+            _ => Err(Error::new(
+                ErrorKind::ImpossibleOperation,
+                "cannot round value",
+            )),
+        }
+    }
+
+    /// Round the number to a given precision.
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
+    pub fn round(_: &State, value: Value, precision: Option<i32>) -> Result<Value, Error> {
+        match value.0 {
+            ValueRepr::I64(_) | ValueRepr::I128(_) => Ok(value),
+            ValueRepr::F64(val) => {
+                let x = 10f64.powi(precision.unwrap_or(0));
+                Ok(Value::from((x * val).round() / x))
+            }
+            _ => Err(Error::new(
+                ErrorKind::ImpossibleOperation,
+                "cannot round value",
+            )),
+        }
+    }
+
+    /// Returns the first item from a list.
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
+    pub fn first(_: &State, value: Value) -> Result<Value, Error> {
+        match value.0 {
+            ValueRepr::String(s) | ValueRepr::SafeString(s) => {
+                Ok(s.chars().next().map_or(Value::UNDEFINED, Value::from))
+            }
+            ValueRepr::Seq(ref s) => Ok(s.first().cloned().unwrap_or(Value::UNDEFINED)),
+            _ => Err(Error::new(
+                ErrorKind::ImpossibleOperation,
+                "cannot get first item from value",
+            )),
+        }
+    }
+
+    /// Returns the last item from a list.
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
+    pub fn last(_: &State, value: Value) -> Result<Value, Error> {
+        match value.0 {
+            ValueRepr::String(s) | ValueRepr::SafeString(s) => {
+                Ok(s.chars().rev().next().map_or(Value::UNDEFINED, Value::from))
+            }
+            ValueRepr::Seq(ref s) => Ok(s.last().cloned().unwrap_or(Value::UNDEFINED)),
+            _ => Err(Error::new(
+                ErrorKind::ImpossibleOperation,
+                "cannot get last item from value",
+            )),
+        }
+    }
+
+    /// Converts the input value into a list.
+    ///
+    /// If the value is already a list, then it's returned unchanged.
+    /// Applied to a map this returns the list of keys, applied to a
+    /// string this returns the characters.  If the value is undefined
+    /// an empty list is returned.
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
+    pub fn list(_: &State, value: Value) -> Result<Value, Error> {
+        match &value.0 {
+            ValueRepr::Undefined => Ok(Value::from(Vec::<Value>::new())),
+            ValueRepr::String(ref s) | ValueRepr::SafeString(ref s) => {
+                Ok(Value::from(s.chars().map(Value::from).collect::<Vec<_>>()))
+            }
+            ValueRepr::Seq(_) => Ok(value.clone()),
+            ValueRepr::Map(ref m) => Ok(Value::from(
+                m.iter()
+                    .map(|x| Value::from(x.0.clone()))
+                    .collect::<Vec<_>>(),
+            )),
+            _ => Err(Error::new(
+                ErrorKind::ImpossibleOperation,
+                "cannot convert value to list",
+            )),
+        }
+    }
+
+    /// """Slice an iterator and return a list of lists containing
+    /// those items. Useful if you want to create a div containing
+    /// three ul tags that represent columns:
+    pub fn slice(
+        _: &State,
+        value: Value,
+        count: usize,
+        fill_with: Option<Value>,
+    ) -> Result<Value, Error> {
+        let items = value.iter().collect::<Vec<_>>();
+        let len = items.len();
+        let items_per_slice = len / count;
+        let slices_with_extra = len % count;
+        let mut offset = 0;
+        let mut rv = Vec::new();
+
+        for slice in 0..count {
+            let start = offset + slice * items_per_slice;
+            if slice < slices_with_extra {
+                offset += 1;
+            }
+            let end = offset + (slice + 1) * items_per_slice;
+            let tmp = &items[start..end];
+
+            if let Some(ref filler) = fill_with {
+                if slice >= slices_with_extra {
+                    let mut tmp = tmp.to_vec();
+                    tmp.push(filler.clone());
+                    rv.push(Value::from(tmp));
+                    continue;
+                }
+            }
+
+            rv.push(Value::from(tmp.to_vec()));
+        }
+
+        Ok(Value::from(rv))
+    }
+
+    /// Batch items.
+    ///
+    /// This filter works pretty much like `slice` just the other way round. It
+    /// returns a list of lists with the given number of items. If you provide a
+    /// second parameter this is used to fill up missing items.
+    pub fn batch(
+        _: &State,
+        value: Value,
+        count: usize,
+        fill_with: Option<Value>,
+    ) -> Result<Value, Error> {
+        let mut rv = Vec::new();
+        let mut tmp = Vec::with_capacity(count);
+
+        for item in value.iter() {
+            if tmp.len() == count {
+                rv.push(Value::from(mem::replace(
+                    &mut tmp,
+                    Vec::with_capacity(count),
+                )));
+            }
+            tmp.push(item);
+        }
+
+        if !tmp.is_empty() {
+            if let Some(filler) = fill_with {
+                for _ in 0..count - tmp.len() {
+                    tmp.push(filler.clone());
+                }
+            }
+            rv.push(Value::from(tmp));
+        }
+
+        Ok(Value::from(rv))
+    }
+
     /// Dumps a value to JSON.
     ///
     /// This filter is only available if the `json` feature is enabled.  The resulting
     /// value is safe to use in HTML as well as it will not contain any special HTML
     /// characters.  The optional parameter to the filter can be set to `true` to enable
     /// pretty printing.
-    #[cfg_attr(docsrs, doc(cfg(all(feature = "builtin_filters", feature = "json"))))]
+    #[cfg_attr(docsrs, doc(cfg(all(feature = "builtins", feature = "json"))))]
     #[cfg(feature = "json")]
     pub fn tojson(_: &State, value: Value, pretty: Option<bool>) -> Result<Value, Error> {
         if pretty.unwrap_or(false) {
@@ -314,42 +510,30 @@ mod builtins {
     /// If given a map it encodes the parameters into a query set, otherwise it
     /// encodes the stringified value.  If the value is none or undefined, an
     /// empty string is returned.
-    #[cfg_attr(
-        docsrs,
-        doc(cfg(all(feature = "builtin_filters", feature = "urlencode")))
-    )]
+    #[cfg_attr(docsrs, doc(cfg(all(feature = "builtins", feature = "urlencode"))))]
     #[cfg(feature = "urlencode")]
     pub fn urlencode(_: &State, value: Value) -> Result<String, Error> {
         const SET: &percent_encoding::AsciiSet =
             &percent_encoding::NON_ALPHANUMERIC.remove(b'/').add(b' ');
-        match (value.as_primitive(), value.kind()) {
-            (Some(Primitive::None), _) | (Some(Primitive::Undefined), _) => Ok("".into()),
-            (Some(Primitive::Bytes(b)), _) => {
-                Ok(percent_encoding::percent_encode(b, SET).to_string())
-            }
-            (Some(Primitive::Str(s)), _) => {
+        match &value.0 {
+            ValueRepr::None | ValueRepr::Undefined => Ok("".into()),
+            ValueRepr::Bytes(b) => Ok(percent_encoding::percent_encode(b, SET).to_string()),
+            ValueRepr::String(s) | ValueRepr::SafeString(s) => {
                 Ok(percent_encoding::utf8_percent_encode(s, SET).to_string())
             }
-            (_, ValueKind::Struct) | (_, ValueKind::Map) => {
+            ValueRepr::Map(ref val) => {
                 let mut rv = String::new();
-                for (idx, ref k) in value.iter().enumerate() {
+                for (idx, (k, v)) in val.iter().enumerate() {
                     if idx > 0 {
                         rv.push('&');
                     }
                     write!(
                         rv,
-                        "{}",
-                        percent_encoding::utf8_percent_encode(&k.to_string(), SET)
+                        "{}={}",
+                        percent_encoding::utf8_percent_encode(&k.to_string(), SET),
+                        percent_encoding::utf8_percent_encode(&v.to_string(), SET)
                     )
                     .unwrap();
-                    if let Ok(v) = value.get_item(k) {
-                        write!(
-                            rv,
-                            "={}",
-                            percent_encoding::utf8_percent_encode(&v.to_string(), SET)
-                        )
-                        .unwrap();
-                    }
                 }
                 Ok(rv)
             }
@@ -424,5 +608,5 @@ mod builtins {
     }
 }
 
-#[cfg(feature = "builtin_filters")]
+#[cfg(feature = "builtins")]
 pub use self::builtins::*;
