@@ -3,6 +3,12 @@ use std::fmt;
 
 use crate::value::Value;
 
+/// This loop has the loop var.
+pub const LOOP_FLAG_WITH_LOOP_VAR: u8 = 1;
+
+/// This loop is recursive.
+pub const LOOP_FLAG_RECURSIVE: u8 = 2;
+
 /// Represents an instruction for the VM.
 #[derive(Clone, PartialEq, Eq)]
 pub enum Instruction<'source> {
@@ -48,6 +54,13 @@ pub enum Instruction<'source> {
     /// Divide the top two values
     Div,
 
+    /// Integer divde the top two values as "integer".
+    ///
+    /// Note that in MiniJinja this currently uses an euclidean
+    /// division to match the rem implementation.  In Python this
+    /// instead uses a flooring division and a flooring remainder.
+    IntDiv,
+
     /// Calculate the remainder the top two values
     Rem,
 
@@ -95,9 +108,8 @@ pub enum Instruction<'source> {
 
     /// Starts a loop
     ///
-    /// The arugment is an indicator if the `loop` variable should
-    /// be available.
-    PushLoop(bool),
+    /// The argument are loop flags.
+    PushLoop(u8),
 
     /// Pushes a value as context layer.
     PushContext,
@@ -159,6 +171,12 @@ pub enum Instruction<'source> {
     /// Discards the top item
     DiscardTop,
 
+    /// A fast super instruction without intermediate capturing.
+    FastSuper,
+
+    /// A fast loop recurse instruction without intermediate capturing.
+    FastRecurse,
+
     /// A nop
     #[allow(unused)]
     Nop,
@@ -182,6 +200,7 @@ impl<'source> fmt::Debug for Instruction<'source> {
             Instruction::Sub => write!(f, "SUB"),
             Instruction::Mul => write!(f, "MUL"),
             Instruction::Div => write!(f, "DIV"),
+            Instruction::IntDiv => write!(f, "INT_DIV"),
             Instruction::Rem => write!(f, "REM"),
             Instruction::Pow => write!(f, "Pow"),
             Instruction::Neg => write!(f, "NEG"),
@@ -201,7 +220,15 @@ impl<'source> fmt::Debug for Instruction<'source> {
                 write!(f, "PERFORM_TEST (name {:?})", n)
             }
             Instruction::Emit => write!(f, "EMIT"),
-            Instruction::PushLoop(v) => write!(f, "PUSH_LOOP (loop var: {:?})", v),
+            Instruction::PushLoop(flags) => {
+                let recursive = flags & LOOP_FLAG_RECURSIVE != 0;
+                let loop_var = flags & LOOP_FLAG_WITH_LOOP_VAR != 0;
+                write!(
+                    f,
+                    "PUSH_LOOP (loop var: {:?}, recursive: {:?})",
+                    loop_var, recursive
+                )
+            }
             Instruction::PushContext => write!(f, "PUSH_CONTEXT"),
             Instruction::Iterate(t) => write!(f, "ITERATE (exit to {:>05x})", t),
             Instruction::PopFrame => write!(f, "POP_FRAME"),
@@ -221,6 +248,8 @@ impl<'source> fmt::Debug for Instruction<'source> {
             Instruction::CallObject => write!(f, "CALL_OBJECT"),
             Instruction::DupTop => write!(f, "DUP_TOP"),
             Instruction::DiscardTop => write!(f, "DISCARD_TOP"),
+            Instruction::FastSuper => write!(f, "FAST_SUPER"),
+            Instruction::FastRecurse => write!(f, "FAST_RECURSE"),
             Instruction::Nop => write!(f, "NOP"),
         }
     }
@@ -315,16 +344,16 @@ impl<'source> Instructions<'source> {
     pub fn get_referenced_names(&self, idx: usize) -> Vec<&'source str> {
         let mut rv = Vec::new();
         for instr in self.instructions[..=idx].iter().rev() {
-            match instr {
-                Instruction::Lookup(name) => rv.push(*name),
-                Instruction::StoreLocal(name) => rv.push(*name),
-                Instruction::CallFunction(name) => rv.push(*name),
-                Instruction::PushLoop(true) => {
-                    rv.push("loop");
-                    break;
-                }
-                Instruction::PushLoop(false) | Instruction::PushContext => break,
-                _ => {}
+            let name = match instr {
+                Instruction::Lookup(name)
+                | Instruction::StoreLocal(name)
+                | Instruction::CallFunction(name) => *name,
+                Instruction::PushLoop(flags) if flags & LOOP_FLAG_WITH_LOOP_VAR != 0 => "loop",
+                Instruction::PushLoop(_) | Instruction::PushContext => break,
+                _ => continue,
+            };
+            if !rv.contains(&name) {
+                rv.push(name);
             }
         }
         rv
